@@ -25,13 +25,13 @@ import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
 
-import javax.servlet.RequestDispatcher;
-import javax.servlet.ServletContainerInitializer;
-import javax.servlet.ServletContext;
-import javax.servlet.ServletRegistration;
-import javax.servlet.ServletRequest;
-import javax.servlet.ServletSecurityElement;
-import javax.servlet.descriptor.JspConfigDescriptor;
+import jakarta.servlet.RequestDispatcher;
+import jakarta.servlet.ServletContainerInitializer;
+import jakarta.servlet.ServletContext;
+import jakarta.servlet.ServletRegistration;
+import jakarta.servlet.ServletRequest;
+import jakarta.servlet.ServletSecurityElement;
+import jakarta.servlet.descriptor.JspConfigDescriptor;
 
 import org.apache.catalina.deploy.NamingResourcesImpl;
 import org.apache.tomcat.ContextBind;
@@ -51,8 +51,8 @@ import org.apache.tomcat.util.http.CookieProcessor;
 /**
  * A <b>Context</b> is a Container that represents a servlet context, and therefore an individual web application, in
  * the Catalina servlet engine. It is therefore useful in almost every deployment of Catalina (even if a Connector
- * attached to a web server such as Apache uses the web server's facilities to identify the appropriate Wrapper to
- * handle this request). It also provides a convenient mechanism to use Interceptors that see every request processed by
+ * attached to a web server (such as Apache) uses the web server's facilities to identify the appropriate Wrapper to
+ * handle this request. It also provides a convenient mechanism to use Interceptors that see every request processed by
  * this particular web application.
  * <p>
  * The parent Container attached to a Context is generally a Host, but may be some other implementation, or may be
@@ -89,7 +89,6 @@ public interface Context extends Container, ContextBind {
      */
     String CHANGE_SESSION_ID_EVENT = "changeSessionId";
 
-
     /**
      * Prefix for resource lookup.
      */
@@ -132,7 +131,7 @@ public interface Context extends Container, ContextBind {
      *
      * @param listeners The set of instantiated listener objects.
      */
-    void setApplicationEventListeners(Object[] listeners);
+    void setApplicationEventListeners(Object listeners[]);
 
 
     /**
@@ -150,7 +149,7 @@ public interface Context extends Container, ContextBind {
      *
      * @param listeners The set of instantiated listener objects.
      */
-    void setApplicationLifecycleListeners(Object[] listeners);
+    void setApplicationLifecycleListeners(Object listeners[]);
 
 
     /**
@@ -246,6 +245,33 @@ public interface Context extends Container, ContextBind {
     void setUseHttpOnly(boolean useHttpOnly);
 
 
+    /**
+     * Should the {@code Partitioned} attribute be added to session cookies created for this web application.
+     * <p>
+     * The name of the attribute used to indicate a partitioned cookie as part of
+     * <a href="https://developers.google.com/privacy-sandbox/3pcd#partitioned">CHIPS</a> is not defined by an RFC and
+     * may change in a non-backwards compatible way once equivalent functionality is included in an RFC.
+     *
+     * @return {@code true} if the {@code Partitioned} attribute should be added to session cookies created for this web
+     *             application, otherwise {@code false}
+     */
+    boolean getUsePartitioned();
+
+
+    /**
+     * Configure whether the {@code Partitioned} attribute should be added to session cookies created for this web
+     * application.
+     * <p>
+     * The name of the attribute used to indicate a partitioned cookie as part of
+     * <a href="https://developers.google.com/privacy-sandbox/3pcd#partitioned">CHIPS</a> is not defined by an RFC and
+     * may change in a non-backwards compatible way once equivalent functionality is included in an RFC.
+     *
+     * @param usePartitioned {@code true} if the {@code Partitioned} attribute should be added to session cookies
+     *                           created for this web application, otherwise {@code false}
+     */
+    void setUsePartitioned(boolean usePartitioned);
+
+
     /**
      * Gets the domain to use for session cookies. Overrides any setting that may be specified by the application.
      *
@@ -291,7 +317,7 @@ public interface Context extends Container, ContextBind {
      * Configures if a / is added to the end of the session cookie path to ensure browsers, particularly IE, don't send
      * a session cookie for context /foo with requests intended for context /foobar.
      *
-     * @param sessionCookiePathUsesTrailingSlash <code>true</code> if the slash should be added, otherwise
+     * @param sessionCookiePathUsesTrailingSlash <code>true</code> if the slash is should be added, otherwise
      *                                               <code>false</code>
      */
     void setSessionCookiePathUsesTrailingSlash(boolean sessionCookiePathUsesTrailingSlash);
@@ -419,6 +445,23 @@ public interface Context extends Container, ContextBind {
     void setIgnoreAnnotations(boolean ignoreAnnotations);
 
 
+    /**
+     * Determine if the metadata of the application is complete. This typically means annotations which add to the
+     * application metadata will be ignored.
+     *
+     * @return {@code true} if metadata is complete for this web application
+     */
+    boolean getMetadataComplete();
+
+
+    /**
+     * Set the boolean on the metadata complete flag for this web application.
+     *
+     * @param metadataComplete The boolean on the metadata complete flag
+     */
+    void setMetadataComplete(boolean metadataComplete);
+
+
     /**
      * @return the login configuration descriptor for this web application.
      */
@@ -717,17 +760,6 @@ public interface Context extends Container, ContextBind {
     String getContainerSciFilter();
 
 
-    /**
-     * @return the value of the parallel annotation scanning flag. If true, it will dispatch scanning to the utility
-     *             executor.
-     *
-     * @deprecated This method will be removed in Tomcat 11 onwards
-     */
-    @Deprecated
-    default boolean isParallelAnnotationScanning() {
-        return getParallelAnnotationScanning();
-    }
-
     /**
      * @return the value of the parallel annotation scanning flag. If true, it will dispatch scanning to the utility
      *             executor.
@@ -920,19 +952,19 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * @return the array of application listener class names configured for this application.
+     * @return the set of application listener class names configured for this application.
      */
     String[] findApplicationListeners();
 
 
     /**
-     * @return the array of application parameters for this application.
+     * @return the set of application parameters for this application.
      */
     ApplicationParameter[] findApplicationParameters();
 
 
     /**
-     * @return the array of security constraints for this web application. If there are none, a zero-length array is
+     * @return the set of security constraints for this web application. If there are none, a zero-length array is
      *             returned.
      */
     SecurityConstraint[] findConstraints();
@@ -946,17 +978,6 @@ public interface Context extends Container, ContextBind {
     ErrorPage findErrorPage(int errorCode);
 
 
-    /**
-     * @param exceptionType Exception type to look up
-     *
-     * @return the error page entry for the specified Java exception type, if any; otherwise return {@code null}.
-     *
-     * @deprecated Unused. Will be removed in Tomcat 10. Use {@link #findErrorPage(Throwable)} instead.
-     */
-    @Deprecated
-    ErrorPage findErrorPage(String exceptionType);
-
-
     /**
      * Find and return the ErrorPage instance for the specified exception's class, or an ErrorPage instance for the
      * closest superclass for which there is such a definition. If no associated ErrorPage instance is found, return
@@ -970,7 +991,7 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * @return the array of defined error pages for all specified error codes and exception types.
+     * @return the set of defined error pages for all specified error codes and exception types.
      */
     ErrorPage[] findErrorPages();
 
@@ -984,13 +1005,13 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * @return the array of defined filters for this Context.
+     * @return the set of defined filters for this Context.
      */
     FilterDef[] findFilterDefs();
 
 
     /**
-     * @return the array of filter mappings for this Context.
+     * @return the set of filter mappings for this Context.
      */
     FilterMap[] findFilterMaps();
 
@@ -1067,28 +1088,6 @@ public interface Context extends Container, ContextBind {
     String[] findServletMappings();
 
 
-    /**
-     * @return the context-relative URI of the error page for the specified HTTP status code, if any; otherwise return
-     *             <code>null</code>.
-     *
-     * @param status HTTP status code to look up
-     *
-     * @deprecated Unused. Will be removed in Tomcat 10. Use {@link #findErrorPage(int)} instead.
-     */
-    @Deprecated
-    String findStatusPage(int status);
-
-
-    /**
-     * @return the set of HTTP status codes for which error pages have been specified. If none are specified, a
-     *             zero-length array is returned.
-     *
-     * @deprecated Unused. Will be removed in Tomcat 10. Use {@link #findErrorPages()} instead.
-     */
-    @Deprecated
-    int[] findStatusPages();
-
-
     /**
      * @return the associated ThreadBindingListener.
      */
@@ -1105,8 +1104,7 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * @return the array of watched resources for this Context. If none are defined, a zero length array will be
-     *             returned.
+     * @return the set of watched resources for this Context. If none are defined, a zero length array will be returned.
      */
     String[] findWatchedResources();
 
@@ -1115,32 +1113,31 @@ public interface Context extends Container, ContextBind {
      * @return <code>true</code> if the specified welcome file is defined for this Context; otherwise return
      *             <code>false</code>.
      *
-     * @param name the welcome file to verify
+     * @param name Welcome file to verify
      */
     boolean findWelcomeFile(String name);
 
 
     /**
-     * @return the array of welcome files defined for this Context. If none are defined, a zero-length array is
-     *             returned.
+     * @return the set of welcome files defined for this Context. If none are defined, a zero-length array is returned.
      */
     String[] findWelcomeFiles();
 
 
     /**
-     * @return the array of LifecycleListener classes that will be added to newly created Wrappers automatically.
+     * @return the set of LifecycleListener classes that will be added to newly created Wrappers automatically.
      */
     String[] findWrapperLifecycles();
 
 
     /**
-     * @return the array of ContainerListener classes that will be added to newly created Wrappers automatically.
+     * @return the set of ContainerListener classes that will be added to newly created Wrappers automatically.
      */
     String[] findWrapperListeners();
 
 
     /**
-     * Notify all {@link javax.servlet.ServletRequestListener}s that a request has started.
+     * Notify all {@link jakarta.servlet.ServletRequestListener}s that a request has started.
      *
      * @param request The request object that will be passed to the listener
      *
@@ -1149,7 +1146,7 @@ public interface Context extends Container, ContextBind {
     boolean fireRequestInitEvent(ServletRequest request);
 
     /**
-     * Notify all {@link javax.servlet.ServletRequestListener}s that a request has ended.
+     * Notify all {@link jakarta.servlet.ServletRequestListener}s that a request has ended.
      *
      * @param request The request object that will be passed to the listener
      *
@@ -1363,7 +1360,7 @@ public interface Context extends Container, ContextBind {
 
     /**
      * Notification that Servlet security has been dynamically set in a
-     * {@link javax.servlet.ServletRegistration.Dynamic}
+     * {@link jakarta.servlet.ServletRegistration.Dynamic}
      *
      * @param registration           Servlet security was modified for
      * @param servletSecurityElement new security constraints for this Servlet
@@ -1661,7 +1658,7 @@ public interface Context extends Container, ContextBind {
 
     /**
      * Controls whether HTTP 1.1 and later location headers generated by a call to
-     * {@link javax.servlet.http.HttpServletResponse#sendRedirect(String)} will use relative or absolute redirects.
+     * {@link jakarta.servlet.http.HttpServletResponse#sendRedirect(String)} will use relative or absolute redirects.
      * <p>
      * Relative redirects are more efficient but may not work with reverse proxies that change the context path. It
      * should be noted that it is not recommended to use a reverse proxy to change the context path because of the
@@ -1676,7 +1673,7 @@ public interface Context extends Container, ContextBind {
 
     /**
      * Will HTTP 1.1 and later location headers generated by a call to
-     * {@link javax.servlet.http.HttpServletResponse#sendRedirect(String)} use relative or absolute redirects.
+     * {@link jakarta.servlet.http.HttpServletResponse#sendRedirect(String)} use relative or absolute redirects.
      *
      * @return {@code true} if relative redirects will be used {@code false} if absolute redirects are used.
      *
@@ -1731,16 +1728,17 @@ public interface Context extends Container, ContextBind {
     String getResponseCharacterEncoding();
 
     /**
-     * Configure if, when returning a context path from {@link javax.servlet.http.HttpServletRequest#getContextPath()},
-     * the return value is allowed to contain multiple leading '/' characters.
+     * Configure if, when returning a context path from
+     * {@link jakarta.servlet.http.HttpServletRequest#getContextPath()}, the return value is allowed to contain multiple
+     * leading '/' characters.
      *
      * @param allowMultipleLeadingForwardSlashInPath The new value for the flag
      */
     void setAllowMultipleLeadingForwardSlashInPath(boolean allowMultipleLeadingForwardSlashInPath);
 
     /**
-     * When returning a context path from {@link javax.servlet.http.HttpServletRequest#getContextPath()}, is it allowed
-     * to contain multiple leading '/' characters?
+     * When returning a context path from {@link jakarta.servlet.http.HttpServletRequest#getContextPath()}, is it
+     * allowed to contain multiple leading '/' characters?
      *
      * @return <code>true</code> if multiple leading '/' characters are allowed, otherwise <code>false</code>
      */
@@ -1772,6 +1770,82 @@ public interface Context extends Container, ContextBind {
     boolean getCreateUploadTargets();
 
 
+    /**
+     * If this is <code>true</code>, every request that is associated with a session will cause the session's last
+     * accessed time to be updated regardless of whether or not the request explicitly accesses the session. If
+     * <code>org.apache.catalina.STRICT_SERVLET_COMPLIANCE</code> is set to <code>true</code>, the default of this
+     * setting will be <code>true</code>, else the default value will be <code>false</code>.
+     *
+     * @return the flag value
+     */
+    boolean getAlwaysAccessSession();
+
+
+    /**
+     * Set the session access behavior.
+     *
+     * @param alwaysAccessSession the new flag value
+     */
+    void setAlwaysAccessSession(boolean alwaysAccessSession);
+
+
+    /**
+     * If this is <code>true</code> then the path passed to <code>ServletContext.getResource()</code> or
+     * <code>ServletContext.getResourceAsStream()</code> must start with &quot;/&quot;. If <code>false</code>, code like
+     * <code>getResource("myfolder/myresource.txt")</code> will work as Tomcat will prepend &quot;/&quot; to the
+     * provided path. If <code>org.apache.catalina.STRICT_SERVLET_COMPLIANCE</code> is set to <code>true</code>, the
+     * default of this setting will be <code>true</code>, else the default value will be <code>false</code>.
+     *
+     * @return the flag value
+     */
+    boolean getContextGetResourceRequiresSlash();
+
+
+    /**
+     * Allow using <code>ServletContext.getResource()</code> or <code>ServletContext.getResourceAsStream()</code>
+     * without a leading &quot;/&quot;.
+     *
+     * @param contextGetResourceRequiresSlash the new flag value
+     */
+    void setContextGetResourceRequiresSlash(boolean contextGetResourceRequiresSlash);
+
+
+    /**
+     * If this is <code>true</code> then any wrapped request or response object passed to an application dispatcher will
+     * be checked to ensure that it has wrapped the original request or response. If
+     * <code>org.apache.catalina.STRICT_SERVLET_COMPLIANCE</code> is set to <code>true</code>, the default of this
+     * setting will be <code>true</code>, else the default value will be <code>false</code>.
+     *
+     * @return the flag value
+     */
+    boolean getDispatcherWrapsSameObject();
+
+
+    /**
+     * Allow disabling the object wrap check in the request dispatcher.
+     *
+     * @param dispatcherWrapsSameObject the new flag value
+     */
+    void setDispatcherWrapsSameObject(boolean dispatcherWrapsSameObject);
+
+
+    /**
+     * If this is <code>true</code>, then following a forward the response will be unwrapped to suspend the Catalina
+     * response instead of simply closing the top level response. The default value is <code>true</code>.
+     *
+     * @return the flag value
+     */
+    boolean getSuspendWrappedResponseAfterForward();
+
+
+    /**
+     * Allows unwrapping the response object to suspend the response following a forward.
+     *
+     * @param suspendWrappedResponseAfterForward the new flag value
+     */
+    void setSuspendWrappedResponseAfterForward(boolean suspendWrappedResponseAfterForward);
+
+
     /**
      * Find configuration file with the specified path, first looking into the webapp resources, then delegating to
      * <code>ConfigFileLoader.getSource().getResource</code>. The <code>WEBAPP_PROTOCOL</code> constant prefix is used
@@ -1801,27 +1875,6 @@ public interface Context extends Container, ContextBind {
     }
 
 
-    /**
-     * @return <code>true</code> if the resources archive lookup will use a bloom filter.
-     *
-     * @deprecated This method will be removed in Tomcat 11 onwards. Use
-     *                 {@link WebResourceRoot#getArchiveIndexStrategy()}
-     */
-    @Deprecated
-    boolean getUseBloomFilterForArchives();
-
-    /**
-     * Set bloom filter flag value.
-     *
-     * @param useBloomFilterForArchives The new fast class path scan flag
-     *
-     * @deprecated This method will be removed in Tomcat 11 onwards Use
-     *                 {@link WebResourceRoot#setArchiveIndexStrategy(String)}
-     */
-    @Deprecated
-    void setUseBloomFilterForArchives(boolean useBloomFilterForArchives);
-
-
     /**
      * Obtain the current configuration for the handling of encoded reverse solidus (%5c - \) characters in paths used
      * to obtain {@link RequestDispatcher} instances for this {@link Context}.
