@@ -19,7 +19,6 @@ package org.apache.catalina.webresources;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
-import java.net.MalformedURLException;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Map;
@@ -28,7 +27,6 @@ import java.util.jar.JarEntry;
 import java.util.jar.JarFile;
 import java.util.jar.JarInputStream;
 import java.util.jar.Manifest;
-import java.util.zip.ZipFile;
 
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.WebResource;
@@ -148,34 +146,11 @@ public class JarWarResourceSet extends AbstractArchiveResourceSet {
                     }
                 }
             }
-            WebResourceRoot root = getRoot();
-            if (root.getArchiveIndexStrategyEnum().getUsesBloom()) {
-                jarContents = new JarContents(archiveEntries.values());
-                retainBloomFilterForArchives = root.getArchiveIndexStrategyEnum().getRetain();
-            }
             return archiveEntries;
         }
     }
 
 
-    /**
-     * {@inheritDoc}
-     * <p>
-     * JarWar needs to generate jarContents for the inner JAR, not the outer WAR.
-     */
-    @Override
-    protected JarFile openJarFile() throws IOException {
-        synchronized (archiveLock) {
-            if (archive == null) {
-                archive = new JarFile(new File(getBase()), true, ZipFile.OPEN_READ, Runtime.version());
-                // Don't populate JarContents here. Populate at the end of getArchiveEntries()
-            }
-            archiveUseCount++;
-            return archive;
-        }
-    }
-
-
     protected void processArchivesEntriesForMultiRelease() {
 
         int targetVersion = Runtime.version().feature();
@@ -259,7 +234,7 @@ public class JarWarResourceSet extends AbstractArchiveResourceSet {
 
         try {
             setBaseUrl(UriUtil.buildJarSafeUrl(new File(getBase())));
-        } catch (MalformedURLException e) {
+        } catch (IOException e) {
             throw new IllegalArgumentException(e);
         }
     }
