@@ -27,6 +27,9 @@ import java.security.cert.CertificateException;
 import java.security.cert.CertificateFactory;
 import java.security.cert.X509Certificate;
 import java.util.ArrayList;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -48,7 +51,6 @@ import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.buf.ByteChunk;
 import org.apache.tomcat.util.buf.MessageBytes;
-import org.apache.tomcat.util.http.Method;
 import org.apache.tomcat.util.http.MimeHeaders;
 import org.apache.tomcat.util.net.AbstractEndpoint.Handler.SocketState;
 import org.apache.tomcat.util.net.ApplicationBufferHandler;
@@ -129,17 +131,29 @@ public class AjpProcessor extends AbstractProcessor {
         System.arraycopy(pongMessage.getBuffer(), 0, pongMessageArray, 0, pongMessage.getLen());
 
         // Build Map of Java Servlet to Jakarta Servlet attribute names
-        jakartaAttributeMapping = Map.of("jakarta.servlet.request.cipher_suite", "jakarta.servlet.request.cipher_suite",
-                "jakarta.servlet.request.key_size", "jakarta.servlet.request.key_size",
-                "jakarta.servlet.request.ssl_session", "jakarta.servlet.request.ssl_session",
-                "jakarta.servlet.request.X509Certificate", "jakarta.servlet.request.X509Certificate",
-                "javax.servlet.request.cipher_suite", "jakarta.servlet.request.cipher_suite",
-                "javax.servlet.request.key_size", "jakarta.servlet.request.key_size",
-                "javax.servlet.request.ssl_session", "jakarta.servlet.request.ssl_session",
-                "javax.servlet.request.X509Certificate", "jakarta.servlet.request.X509Certificate");
+        Map<String,String> m = new HashMap<>();
+        m.put("jakarta.servlet.request.secure_protocol", "jakarta.servlet.request.secure_protocol");
+        m.put("jakarta.servlet.request.cipher_suite", "jakarta.servlet.request.cipher_suite");
+        m.put("jakarta.servlet.request.key_size", "jakarta.servlet.request.key_size");
+        m.put("jakarta.servlet.request.ssl_session", "jakarta.servlet.request.ssl_session");
+        m.put("jakarta.servlet.request.X509Certificate", "jakarta.servlet.request.X509Certificate");
+        m.put("javax.servlet.request.cipher_suite", "jakarta.servlet.request.cipher_suite");
+        m.put("javax.servlet.request.key_size", "jakarta.servlet.request.key_size");
+        m.put("javax.servlet.request.ssl_session", "jakarta.servlet.request.ssl_session");
+        m.put("javax.servlet.request.X509Certificate", "jakarta.servlet.request.X509Certificate");
+        jakartaAttributeMapping = Collections.unmodifiableMap(m);
 
-        iisTlsAttributes = Set.of("CERT_ISSUER", "CERT_SUBJECT", "CERT_COOKIE", "HTTPS_SERVER_SUBJECT", "CERT_FLAGS",
-                "HTTPS_SECRETKEYSIZE", "CERT_SERIALNUMBER", "HTTPS_SERVER_ISSUER", "HTTPS_KEYSIZE");
+        Set<String> s = new HashSet<>();
+        s.add("CERT_ISSUER");
+        s.add("CERT_SUBJECT");
+        s.add("CERT_COOKIE");
+        s.add("HTTPS_SERVER_SUBJECT");
+        s.add("CERT_FLAGS");
+        s.add("HTTPS_SECRETKEYSIZE");
+        s.add("CERT_SERIALNUMBER");
+        s.add("HTTPS_SERVER_ISSUER");
+        s.add("HTTPS_KEYSIZE");
+        iisTlsAttributes = Collections.unmodifiableSet(s);
     }
 
 
@@ -361,11 +375,11 @@ public class AjpProcessor extends AbstractProcessor {
                     try {
                         socketWrapper.write(true, pongMessageArray, 0, pongMessageArray.length);
                         socketWrapper.flush(true);
-                    } catch (IOException ioe) {
+                    } catch (IOException e) {
                         if (getLog().isDebugEnabled()) {
-                            getLog().debug(sm.getString("ajpprocessor.pongFail"), ioe);
+                            getLog().debug(sm.getString("ajpprocessor.pongFail"), e);
                         }
-                        setErrorState(ErrorState.CLOSE_CONNECTION_NOW, ioe);
+                        setErrorState(ErrorState.CLOSE_CONNECTION_NOW, e);
                     }
                     recycle();
                     continue;
@@ -378,15 +392,13 @@ public class AjpProcessor extends AbstractProcessor {
                     setErrorState(ErrorState.CLOSE_CONNECTION_NOW, null);
                     break;
                 }
-                request.markStartTime();
-            } catch (IOException ioe) {
-                setErrorState(ErrorState.CLOSE_CONNECTION_NOW, ioe);
+                request.setStartTimeNanos(System.nanoTime());
+            } catch (IOException e) {
+                setErrorState(ErrorState.CLOSE_CONNECTION_NOW, e);
                 break;
             } catch (Throwable t) {
                 ExceptionUtils.handleThrowable(t);
-                if (getLog().isDebugEnabled()) {
-                    getLog().debug(sm.getString("ajpprocessor.header.error"), t);
-                }
+                getLog().debug(sm.getString("ajpprocessor.header.error"), t);
                 // 400 - Bad Request
                 response.setStatus(400);
                 setErrorState(ErrorState.CLOSE_CLEAN, t);
@@ -399,9 +411,7 @@ public class AjpProcessor extends AbstractProcessor {
                     prepareRequest();
                 } catch (Throwable t) {
                     ExceptionUtils.handleThrowable(t);
-                    if (getLog().isDebugEnabled()) {
-                        getLog().debug(sm.getString("ajpprocessor.request.prepare"), t);
-                    }
+                    getLog().debug(sm.getString("ajpprocessor.request.prepare"), t);
                     // 500 - Internal Server Error
                     response.setStatus(500);
                     setErrorState(ErrorState.CLOSE_CLEAN, t);
@@ -567,7 +577,7 @@ public class AjpProcessor extends AbstractProcessor {
             // Zero length message.
             return true;
         } else {
-            if (messageLength > (buf.length - Constants.H_SIZE)) {
+            if (messageLength > message.getBuffer().length) {
                 // Message too long for the buffer
                 // Need to trigger a 400 response
                 String msg = sm.getString("ajpprocessor.header.tooLong", Integer.valueOf(messageLength),
@@ -634,13 +644,14 @@ public class AjpProcessor extends AbstractProcessor {
     /**
      * After reading the request headers, we have to setup the request filters.
      */
+    @SuppressWarnings("deprecation")
     private void prepareRequest() {
 
         // Translate the HTTP method code to a String.
         byte methodCode = requestHeaderMessage.getByte();
         if (methodCode != Constants.SC_M_JK_STORED) {
             String methodName = Constants.getMethodForCode(methodCode - 1);
-            request.setMethod(methodName);
+            request.method().setString(methodName);
         }
 
         requestHeaderMessage.getBytes(request.protocol());
@@ -669,7 +680,7 @@ public class AjpProcessor extends AbstractProcessor {
         boolean contentLengthSet = false;
         int hCount = requestHeaderMessage.getInt();
         for (int i = 0; i < hCount; i++) {
-            String hName;
+            String hName = null;
 
             // Header names are encoded as either an integer code starting
             // with 0xA0, or as a normal string (in which case the first
@@ -677,7 +688,7 @@ public class AjpProcessor extends AbstractProcessor {
             int isc = requestHeaderMessage.peekInt();
             int hId = isc & 0xFF;
 
-            MessageBytes vMB;
+            MessageBytes vMB = null;
             isc &= 0xFF00;
             if (0xA000 == isc) {
                 requestHeaderMessage.getInt(); // To advance the read position
@@ -742,6 +753,7 @@ public class AjpProcessor extends AbstractProcessor {
                             // Ignore invalid value
                         }
                     } else if (n.equals(Constants.SC_A_SSL_PROTOCOL)) {
+                        request.setAttribute(SSLSupport.SECURE_PROTOCOL_KEY, v);
                         request.setAttribute(SSLSupport.PROTOCOL_VERSION_KEY, v);
                     } else if (n.equals("JK_LB_ACTIVATION")) {
                         request.setAttribute(n, v);
@@ -827,14 +839,12 @@ public class AjpProcessor extends AbstractProcessor {
                     break;
 
                 case Constants.SC_A_STORED_METHOD:
-                    requestHeaderMessage.getBytes(tmpMB);
-                    ByteChunk tmpBC = tmpMB.getByteChunk();
-                    request.setMethod(tmpBC.getBytes(), tmpBC.getStart(), tmpBC.getLength());
+                    requestHeaderMessage.getBytes(request.method());
                     break;
 
                 case Constants.SC_A_SECRET:
                     requestHeaderMessage.getBytes(tmpMB);
-                    if (secret != null && !secret.isEmpty()) {
+                    if (secret != null && secret.length() > 0) {
                         secretPresentInRequest = true;
                         if (!tmpMB.equals(secret)) {
                             response.setStatus(403);
@@ -852,7 +862,7 @@ public class AjpProcessor extends AbstractProcessor {
         }
 
         // Check if secret was submitted if required
-        if (secret != null && !secret.isEmpty() && !secretPresentInRequest) {
+        if (secret != null && secret.length() > 0 && !secretPresentInRequest) {
             response.setStatus(403);
             setErrorState(ErrorState.CLOSE_CLEAN, null);
         }
@@ -863,9 +873,10 @@ public class AjpProcessor extends AbstractProcessor {
 
             int pos = uriBC.indexOf("://", 0, 3, 4);
             int uriBCStart = uriBC.getStart();
+            int slashPos = -1;
             if (pos != -1) {
                 byte[] uriB = uriBC.getBytes();
-                int slashPos = uriBC.indexOf('/', pos + 3);
+                slashPos = uriBC.indexOf('/', pos + 3);
                 if (slashPos == -1) {
                     slashPos = uriBC.getLength();
                     // Set URI as "/"
@@ -897,9 +908,9 @@ public class AjpProcessor extends AbstractProcessor {
     protected void populateHost() {
         try {
             request.serverName().duplicate(request.localName());
-        } catch (IOException ioe) {
+        } catch (IOException e) {
             response.setStatus(400);
-            setErrorState(ErrorState.CLOSE_CLEAN, ioe);
+            setErrorState(ErrorState.CLOSE_CLEAN, e);
         }
     }
 
@@ -927,7 +938,7 @@ public class AjpProcessor extends AbstractProcessor {
         // Responses with certain status codes and/or methods are not permitted to include a response body.
         int statusCode = response.getStatus();
         if (statusCode < 200 || statusCode == 204 || statusCode == 205 || statusCode == 304 ||
-                Method.HEAD.equals(request.getMethod())) {
+                request.method().equals("HEAD")) {
             // No entity body
             swallowResponse = true;
         }
@@ -1060,12 +1071,10 @@ public class AjpProcessor extends AbstractProcessor {
         if (empty && doRead) {
             try {
                 refillReadBuffer(false);
-            } catch (IOException ioe) {
-                /*
-                 * Probably a timeout. This approach isn't ideal but it works. Returning 1 will indicate that data is
-                 * available which should trigger a read which in turn will trigger another IOException and that one can
-                 * be thrown.
-                 */
+            } catch (IOException timeout) {
+                // Not ideal. This will indicate that data is available
+                // which should trigger a read which in turn will trigger
+                // another IOException and that one can be thrown.
                 return 1;
             }
         }
@@ -1307,8 +1316,8 @@ public class AjpProcessor extends AbstractProcessor {
                 // Validate and write response headers
                 try {
                     prepareResponse();
-                } catch (IOException ioe) {
-                    setErrorState(ErrorState.CLOSE_CONNECTION_NOW, ioe);
+                } catch (IOException e) {
+                    setErrorState(ErrorState.CLOSE_CONNECTION_NOW, e);
                 }
             }
 
