@@ -48,7 +48,6 @@ import org.apache.coyote.http2.Http2Parser.Input;
 import org.apache.coyote.http2.Http2Parser.Output;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
-import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.http.MimeHeaders;
 import org.apache.tomcat.util.http.parser.Priority;
 import org.apache.tomcat.util.log.UserDataHelper;
@@ -65,7 +64,7 @@ import org.apache.tomcat.util.res.StringManager;
  * For reading, this implementation is blocking within frames and non-blocking between frames. <br>
  * Note:
  * <ul>
- * <li>You will need to nest a &lt;UpgradeProtocol className="org.apache.coyote.http2.Http2Protocol" /&gt; element
+ * <li>You will need to nest an &lt;UpgradeProtocol className="org.apache.coyote.http2.Http2Protocol" /&gt; element
  * inside a TLS enabled Connector element in server.xml to enable HTTP/2 support.</li>
  * </ul>
  */
@@ -102,7 +101,7 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
     private volatile Http2Parser parser;
 
     // Simple state machine (sequence of states)
-    private final AtomicReference<ConnectionState> connectionState = new AtomicReference<>(ConnectionState.NEW);
+    private AtomicReference<ConnectionState> connectionState = new AtomicReference<>(ConnectionState.NEW);
     private volatile long pausedNanoTime = Long.MAX_VALUE;
 
     /**
@@ -122,7 +121,6 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
     private final ConcurrentNavigableMap<Integer,AbstractNonZeroStream> streams = new ConcurrentSkipListMap<>();
     protected final AtomicInteger activeRemoteStreamCount = new AtomicInteger(0);
     private volatile int maxProcessedStreamId;
-    private final AtomicInteger nextLocalStreamId = new AtomicInteger(2);
     private final PingManager pingManager = getPingManager();
     private volatile int newStreamsSinceLastPrune = 0;
     private final Set<Stream> backLogStreams = new HashSet<>();
@@ -154,7 +152,7 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
         // Over time the count should be a slowly decreasing negative number.
         // Therefore, the longer a connection is 'well-behaved', the greater
         // tolerance it will have for a period of 'bad' behaviour.
-        overheadCount = new AtomicLong(-10L * protocol.getOverheadCountFactor());
+        overheadCount = new AtomicLong(-10 * protocol.getOverheadCountFactor());
 
         lastNonFinalDataPayload = protocol.getOverheadDataThreshold() * 2;
         lastWindowUpdate = protocol.getOverheadWindowUpdateThreshold() * 2;
@@ -164,12 +162,8 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
         remoteSettings = new ConnectionSettingsRemote(connectionId);
         localSettings = new ConnectionSettingsLocal(connectionId);
 
-        /*
-         * Force set these initial limits. A well-behaved client should ACK the settings and adhere to them before it
-         * reaches the limits anyway.
-         */
-        localSettings.set(Setting.MAX_CONCURRENT_STREAMS, protocol.getMaxConcurrentStreams(), true);
-        localSettings.set(Setting.INITIAL_WINDOW_SIZE, protocol.getInitialWindowSize(), true);
+        localSettings.set(Setting.MAX_CONCURRENT_STREAMS, protocol.getMaxConcurrentStreams());
+        localSettings.set(Setting.INITIAL_WINDOW_SIZE, protocol.getInitialWindowSize());
 
         pingManager.initiateDisabled = protocol.getInitiatePingDisabled();
 
@@ -447,13 +441,6 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
                 log.debug(sm.getString("upgradeHandler.ioerror", connectionId), ioe);
             }
             close();
-        } catch (Throwable t) {
-            ExceptionUtils.handleThrowable(t);
-            if (log.isDebugEnabled()) {
-                log.debug(sm.getString("upgradeHandler.throwable", connectionId), t);
-            }
-            // Unexpected errors close the connection.
-            close();
         }
 
         if (log.isTraceEnabled()) {
@@ -526,7 +513,7 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
 
             try {
                 writeGoAwayFrame((1 << 31) - 1, Http2Error.NO_ERROR.getCode(), null);
-            } catch (IOException ignore) {
+            } catch (IOException ioe) {
                 // This is fatal for the connection. Ignore it here. There will be
                 // further attempts at I/O in upgradeDispatch() and it can better
                 // handle the IO errors.
@@ -585,9 +572,6 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
                     se.getError(), se.getMessage()));
         }
 
-        // Treat a sent reset like a received reset and increment the overhead count
-        increaseOverheadCount(FrameType.RST, getProtocol().getOverheadResetFactor());
-
         // Write a RST frame
         byte[] rstFrame = new byte[13];
         // Length
@@ -636,7 +620,7 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
         }
         try {
             writeGoAwayFrame(maxProcessedStreamId, code, msg);
-        } catch (IOException ignore) {
+        } catch (IOException ioe) {
             // Ignore. GOAWAY is sent on a best efforts basis and the original
             // error has already been logged.
         }
@@ -660,7 +644,6 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
             }
             socketWrapper.flush(true);
         } catch (IOException ioe) {
-            // Exception is logged further up stack
             String msg = sm.getString("upgradeHandler.sendPrefaceFail", connectionId);
             if (log.isDebugEnabled()) {
                 log.debug(msg);
@@ -718,13 +701,12 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
         }
     }
 
-    void writeHeaders(Stream stream, int pushedStreamId, MimeHeaders mimeHeaders, boolean endOfStream, int payloadSize)
-            throws IOException {
+    void writeHeaders(Stream stream, MimeHeaders mimeHeaders, boolean endOfStream, int payloadSize) throws IOException {
         // This ensures the Stream processing thread has control of the socket.
         Lock lock = socketWrapper.getLock();
         lock.lock();
         try {
-            doWriteHeaders(stream, pushedStreamId, mimeHeaders, endOfStream, payloadSize);
+            doWriteHeaders(stream, mimeHeaders, endOfStream, payloadSize);
         } finally {
             lock.unlock();
         }
@@ -739,17 +721,12 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
      * Separate method to allow Http2AsyncUpgradeHandler to call this code without synchronizing on socketWrapper since
      * it doesn't need to.
      */
-    protected HeaderFrameBuffers doWriteHeaders(Stream stream, int pushedStreamId, MimeHeaders mimeHeaders,
-            boolean endOfStream, int payloadSize) throws IOException {
+    protected HeaderFrameBuffers doWriteHeaders(Stream stream, MimeHeaders mimeHeaders, boolean endOfStream,
+            int payloadSize) throws IOException {
 
         if (log.isTraceEnabled()) {
-            if (pushedStreamId == 0) {
-                log.trace(sm.getString("upgradeHandler.writeHeaders", connectionId, stream.getIdAsString(),
-                        Boolean.valueOf(endOfStream)));
-            } else {
-                log.trace(sm.getString("upgradeHandler.writePushHeaders", connectionId, stream.getIdAsString(),
-                        Integer.valueOf(pushedStreamId), Boolean.valueOf(endOfStream)));
-            }
+            log.trace(sm.getString("upgradeHandler.writeHeaders", connectionId, stream.getIdAsString(),
+                    Boolean.valueOf(endOfStream)));
         }
 
         if (!stream.canWrite()) {
@@ -758,31 +735,18 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
 
         HeaderFrameBuffers headerFrameBuffers = getHeaderFrameBuffers(payloadSize);
 
-        byte[] pushedStreamIdBytes = null;
-        if (pushedStreamId > 0) {
-            pushedStreamIdBytes = new byte[4];
-            ByteUtil.set31Bits(pushedStreamIdBytes, 0, pushedStreamId);
-        }
-
         boolean first = true;
         State state = null;
 
         while (state != State.COMPLETE) {
             headerFrameBuffers.startFrame();
-            if (first && pushedStreamIdBytes != null) {
-                headerFrameBuffers.getPayload().put(pushedStreamIdBytes);
-            }
             state = getHpackEncoder().encode(mimeHeaders, headerFrameBuffers.getPayload());
             headerFrameBuffers.getPayload().flip();
             if (state == State.COMPLETE || headerFrameBuffers.getPayload().limit() > 0) {
                 ByteUtil.setThreeBytes(headerFrameBuffers.getHeader(), 0, headerFrameBuffers.getPayload().limit());
                 if (first) {
                     first = false;
-                    if (pushedStreamIdBytes == null) {
-                        headerFrameBuffers.getHeader()[3] = FrameType.HEADERS.getIdByte();
-                    } else {
-                        headerFrameBuffers.getHeader()[3] = FrameType.PUSH_PROMISE.getIdByte();
-                    }
+                    headerFrameBuffers.getHeader()[3] = FrameType.HEADERS.getIdByte();
                     if (endOfStream) {
                         headerFrameBuffers.getHeader()[4] = FLAG_END_OF_STREAM;
                     }
@@ -972,8 +936,8 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
                 } else if (windowSize < 1) {
                     /*
                      * The connection window has no capacity. If the stream has not been granted an allocation, and the
-                     * stream was not already added to the backlog due to a partial reservation (see next else if block)
-                     * add it to the backlog so it can obtain an allocation when capacity is available.
+                     * stream was not already added to the backlog due to an partial reservation (see next else if
+                     * block) add it to the backlog so it can obtain an allocation when capacity is available.
                      */
                     if (stream.getConnectionAllocationMade() == 0 && stream.getConnectionAllocationRequested() == 0) {
                         stream.setConnectionAllocationRequested(reservation);
@@ -1219,7 +1183,12 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
             int leftToAllocate = allocation;
 
             if (stream.getConnectionAllocationRequested() > 0) {
-                int allocatedThisTime = Math.min(allocation, stream.getConnectionAllocationRequested());
+                int allocatedThisTime;
+                if (allocation >= stream.getConnectionAllocationRequested()) {
+                    allocatedThisTime = stream.getConnectionAllocationRequested();
+                } else {
+                    allocatedThisTime = allocation;
+                }
                 stream.setConnectionAllocationRequested(stream.getConnectionAllocationRequested() - allocatedThisTime);
                 stream.setConnectionAllocationMade(stream.getConnectionAllocationMade() + allocatedThisTime);
                 leftToAllocate = leftToAllocate - allocatedThisTime;
@@ -1291,17 +1260,6 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
     }
 
 
-    private Stream createLocalStream(Request request) {
-        int streamId = nextLocalStreamId.getAndAdd(2);
-
-        Integer key = Integer.valueOf(streamId);
-
-        Stream result = new Stream(key, this, request);
-        streams.put(key, result);
-        return result;
-    }
-
-
     private void close() {
         ConnectionState previous = connectionState.getAndSet(ConnectionState.CLOSED);
         if (previous == ConnectionState.CLOSED) {
@@ -1381,36 +1339,6 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
     }
 
 
-    void push(Request request, Stream associatedStream) throws IOException {
-        if (localSettings.getMaxConcurrentStreams() < activeRemoteStreamCount.incrementAndGet()) {
-            // If there are too many open streams, simply ignore the push
-            // request.
-            setConnectionTimeoutForStreamCount(activeRemoteStreamCount.decrementAndGet());
-            return;
-        }
-
-        Stream pushStream;
-
-        /*
-         * Uses SocketWrapper lock since PUSH_PROMISE frames have to be sent in order. Once the stream has been created
-         * we need to ensure that the PUSH_PROMISE is sent before the next stream is created for a PUSH_PROMISE.
-         */
-        Lock lock = socketWrapper.getLock();
-        lock.lock();
-        try {
-            pushStream = createLocalStream(request);
-            writeHeaders(associatedStream, pushStream.getIdAsInt(), request.getMimeHeaders(), false,
-                    Constants.DEFAULT_HEADERS_FRAME_SIZE);
-        } finally {
-            lock.unlock();
-        }
-
-        pushStream.sentPushPromise();
-
-        processStreamOnContainerThread(pushStream);
-    }
-
-
     @Override
     protected final String getConnectionId() {
         return connectionId;
@@ -1418,59 +1346,39 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
 
 
     void reduceOverheadCount(FrameType frameType) {
-        /*
-         * A non-overhead frame reduces the overhead count by {@code Http2Protocol.DEFAULT_OVERHEAD_REDUCTION_FACTOR}.
-         *
-         * A simple browser request is likely to have one non-overhead frame (HEADERS) that results in a response with
-         * one further non-overhead frame (DATA). With the default settings, the overhead count will reduce by 40 for
-         * each simple request.
-         *
-         * Requests and responses with bodies will create additional non-overhead frames, further reducing the overhead
-         * count.
-         */
+        // A non-overhead frame reduces the overhead count by
+        // Http2Protocol.DEFAULT_OVERHEAD_REDUCTION_FACTOR. A simple browser
+        // request is likely to have one non-overhead frame (HEADERS) and one
+        // overhead frame (REPRIORITISE). With the default settings the overhead
+        // count will reduce by 10 for each simple request.
+        // Requests and responses with bodies will create additional
+        // non-overhead frames, further reducing the overhead count.
         updateOverheadCount(frameType, Http2Protocol.DEFAULT_OVERHEAD_REDUCTION_FACTOR);
     }
 
 
     @Override
     public void increaseOverheadCount(FrameType frameType) {
-        /*
-         * An overhead frame (SETTINGS, PRIORITY, PING) increases the overhead count by overheadCountFactor. By default,
-         * this means an overhead frame increases the overhead count by 10.
-         *
-         * If the client ignores maxConcurrentStreams then any HEADERS frame received will also increase the overhead
-         * count by overheadCountFactor.
-         *
-         * A simple browser request should not trigger any overhead frames.
-         */
+        // An overhead frame increases the overhead count by
+        // overheadCountFactor. By default, this means an overhead frame
+        // increases the overhead count by 10. A simple browser request is
+        // likely to have one non-overhead frame (HEADERS) and one overhead
+        // frame (REPRIORITISE). With the default settings the overhead count
+        // will reduce by 10 for each simple request.
         updateOverheadCount(frameType, getProtocol().getOverheadCountFactor());
     }
 
 
-    /**
-     * Used to increase the overhead for frames that don't use the {@code overheadCountFactor} ({@code CONTINUATION},
-     * {@code DATA}, {@code WINDOW_UPDATE} and {@code RESET}).
-     *
-     * @param frameType The frame type triggering the overhead increase
-     * @param increment The amount by which the overhead is increased
-     */
-    protected void increaseOverheadCount(FrameType frameType, int increment) {
-        /*
-         * Three types of frame are susceptible to inefficient (and potentially malicious) use of small frames. These
-         * trigger an increase in overhead that is inversely proportional to size. The default threshold for all three
-         * potential areas for abuse (CONTINUATION, DATA, WINDOW_UPDATE) is 1024 bytes. Frames with sizes smaller than
-         * this will trigger an increase of threshold/size.
-         *
-         * The check for DATA and WINDOW_UPDATE frames takes an average over the last two frames to allow for client
-         * buffering schemes that can result in some small DATA payloads.
-         *
-         * The CONTINUATION and DATA frames checks are skipped for end of headers (CONTINUATION) and end of stream
-         * (DATA) as those frames may be small for legitimate reasons.
-         *
-         * RESET frames (received or sent) trigger an increase of overheadResetFactor.
-         *
-         * In all cases, the calling method determines the extent to which the overhead count is increased.
-         */
+    private void increaseOverheadCount(FrameType frameType, int increment) {
+        // Overhead frames that indicate inefficient (and potentially malicious)
+        // use of small frames trigger an increase that is inversely
+        // proportional to size. The default threshold for all three potential
+        // areas for abuse (HEADERS, DATA, WINDOW_UPDATE) is 1024 bytes. Frames
+        // with sizes smaller than this will trigger an increase of
+        // threshold/size.
+        // DATA and WINDOW_UPDATE take an average over the last two non-final
+        // frames to allow for client buffering schemes that can result in some
+        // small DATA payloads.
         updateOverheadCount(frameType, increment);
     }
 
@@ -1496,7 +1404,7 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
         int len = length;
         int pos = offset;
         boolean nextReadBlock = block;
-        int thisRead;
+        int thisRead = 0;
 
         while (len > 0) {
             // Blocking reads use the protocol level read timeout. Non-blocking
@@ -1551,7 +1459,7 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
 
 
     @Override
-    public ByteBuffer startRequestBodyFrame(int streamId, int dataLength, boolean endOfStream) throws Http2Exception {
+    public ByteBuffer startRequestBodyFrame(int streamId, int payloadSize, boolean endOfStream) throws Http2Exception {
         // DATA frames reduce the overhead count ...
         reduceOverheadCount(FrameType.DATA);
 
@@ -1565,8 +1473,8 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
         // average over two frames to avoid false positives.
         if (!endOfStream) {
             int overheadThreshold = protocol.getOverheadDataThreshold();
-            int average = (lastNonFinalDataPayload >> 1) + (dataLength >> 1);
-            lastNonFinalDataPayload = dataLength;
+            int average = (lastNonFinalDataPayload >> 1) + (payloadSize >> 1);
+            lastNonFinalDataPayload = payloadSize;
             // Avoid division by zero
             if (average == 0) {
                 average = 1;
@@ -1578,7 +1486,7 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
 
         AbstractNonZeroStream abstractNonZeroStream = getAbstractNonZeroStream(streamId, true);
         abstractNonZeroStream.checkState(FrameType.DATA);
-        abstractNonZeroStream.receivedData(dataLength);
+        abstractNonZeroStream.receivedData(payloadSize);
         ByteBuffer result = abstractNonZeroStream.getInputByteBuffer(true);
 
         if (log.isTraceEnabled()) {
@@ -1651,24 +1559,6 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
     }
 
 
-    /**
-     * Unused - NO-OP.
-     *
-     * @param streamId       Unused
-     * @param parentStreamId Unused
-     * @param exclusive      Unused
-     * @param weight         Unused
-     *
-     * @throws Http2Exception Never thrown
-     *
-     * @deprecated Unused. Will be removed in Tomcat 11 onwards.
-     */
-    @Deprecated
-    public void reprioritise(int streamId, int parentStreamId, boolean exclusive, int weight) throws Http2Exception {
-        // NO-OP
-    }
-
-
     @Override
     public void headersContinue(int payloadSize, boolean endOfHeaders) {
         // Generally, continuation frames don't impact the overhead count but if
@@ -1679,9 +1569,9 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
             if (payloadSize < overheadThreshold) {
                 if (payloadSize == 0) {
                     // Avoid division by zero
-                    increaseOverheadCount(FrameType.CONTINUATION, overheadThreshold);
+                    increaseOverheadCount(FrameType.HEADERS, overheadThreshold);
                 } else {
-                    increaseOverheadCount(FrameType.CONTINUATION, overheadThreshold / payloadSize);
+                    increaseOverheadCount(FrameType.HEADERS, overheadThreshold / payloadSize);
                 }
             }
         }
@@ -1949,7 +1839,7 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
         // 10 seconds
         protected final long pingIntervalNano = 10000000000L;
 
-        protected volatile int sequence = 0;
+        protected int sequence = 0;
         protected long lastPingNanoTime = Long.MIN_VALUE;
 
         protected Queue<PingRecord> inflightPings = new ConcurrentLinkedQueue<>();
@@ -1970,13 +1860,18 @@ class Http2UpgradeHandler extends AbstractStream implements InternalHttpUpgradeH
             if (force || now - lastPingNanoTime > pingIntervalNano) {
                 lastPingNanoTime = now;
                 byte[] payload = new byte[8];
-                int sentSequence = ++sequence;
-                PingRecord pingRecord = new PingRecord(sentSequence, now);
-                inflightPings.add(pingRecord);
-                ByteUtil.set31Bits(payload, 4, sentSequence);
-                socketWrapper.write(true, PING, 0, PING.length);
-                socketWrapper.write(true, payload, 0, payload.length);
-                socketWrapper.flush(true);
+                socketWrapper.getLock().lock();
+                try {
+                    int sentSequence = ++sequence;
+                    PingRecord pingRecord = new PingRecord(sentSequence, now);
+                    inflightPings.add(pingRecord);
+                    ByteUtil.set31Bits(payload, 4, sentSequence);
+                    socketWrapper.write(true, PING, 0, PING.length);
+                    socketWrapper.write(true, payload, 0, payload.length);
+                    socketWrapper.flush(true);
+                } finally {
+                    socketWrapper.getLock().unlock();
+                }
             }
         }
 
