@@ -36,7 +36,12 @@ import org.xml.sax.helpers.AttributesImpl;
 /**
  * This class implements a parser for a JSP page (non-xml view). JSP page grammar is included here for reference. The
  * token '#' that appears in the production indicates the current input token location in the production.
+ *
+ * @author Kin-man Chung
+ * @author Shawn Bayern
+ * @author Mark Roth
  */
+
 class Parser implements TagConstants {
 
     private final ParserController parserController;
@@ -63,8 +68,6 @@ class Parser implements TagConstants {
     // These are not accessible from outside the parser.
     private static final String JAVAX_BODY_CONTENT_PARAM = "JAVAX_BODY_CONTENT_PARAM";
 
-    private static final String JAVAX_BODY_CONTENT_PLUGIN = "JAVAX_BODY_CONTENT_PLUGIN";
-
     private static final String JAVAX_BODY_CONTENT_TEMPLATE_TEXT = "JAVAX_BODY_CONTENT_TEMPLATE_TEXT";
 
     /**
@@ -131,7 +134,8 @@ class Parser implements TagConstants {
             parser.addInclude(root, pageInfo.getIncludeCoda());
         }
 
-        return new Node.Nodes(root);
+        Node.Nodes page = new Node.Nodes(root);
+        return page;
     }
 
     /**
@@ -238,12 +242,15 @@ class Parser implements TagConstants {
         char ch = (char) reader.peekChar();
         if (Character.isLetter(ch) || ch == '_' || ch == ':') {
             StringBuilder buf = new StringBuilder();
-            do {
+            buf.append(ch);
+            reader.nextChar();
+            ch = (char) reader.peekChar();
+            while (Character.isLetter(ch) || Character.isDigit(ch) || ch == '.' || ch == '_' || ch == '-' ||
+                    ch == ':') {
                 buf.append(ch);
                 reader.nextChar();
                 ch = (char) reader.peekChar();
-            } while (Character.isLetter(ch) || Character.isDigit(ch) || ch == '.' || ch == '_' || ch == '-' ||
-                    ch == ':');
+            }
             return buf.toString();
         }
         return null;
@@ -325,8 +332,8 @@ class Parser implements TagConstants {
             }
         } catch (FileNotFoundException ex) {
             err.jspError(start, "jsp.error.file.not.found", file);
-        } catch (Exception e) {
-            err.jspError(start, e.getMessage());
+        } catch (Exception ex) {
+            err.jspError(start, ex.getMessage());
         }
     }
 
@@ -559,7 +566,7 @@ class Parser implements TagConstants {
     }
 
     /*
-     * Parses an attribute directive with the following syntax: AttributeDirective ::= ( S Attribute)*
+     * Parses a attribute directive with the following syntax: AttributeDirective ::= ( S Attribute)*
      */
     private void parseAttributeDirective(Node parent) throws JasperException {
         Attributes attrs = parseAttributes();
@@ -902,11 +909,8 @@ class Parser implements TagConstants {
     }
 
     /*
-     * <<<<<<< HEAD Parses OptionalBody, but also reused to parse bodies for plugin and param since the syntax is
-     * identical (the only thing that differs substantially is how to process the body, and thus we accept the body type
-     * as a parameter). ======= Parses OptionalBody, but also reused to parse bodies for param since the syntax is
-     * identical (the only thing that differs substantially is how to process the body, and thus we accept the body type
-     * as a parameter). >>>>>>> 76d2ad0a5d (Code clean-up. Formatting. No functional change.)
+     * Parses OptionalBody, but also reused to parse bodies for param since the syntax is identical (the only thing that
+     * differs substantially is how to process the body, and thus we accept the body type as a parameter).
      *
      * OptionalBody ::= EmptyBody | ActionBody
      *
@@ -955,7 +959,8 @@ class Parser implements TagConstants {
         boolean result = false;
 
         if (reader.matchesOptionalSpacesFollowedBy("<jsp:attribute")) {
-            // It may be an EmptyBody, depending on whether there's a "<jsp:body" before the ETag
+            // May be an EmptyBody, depending on whether
+            // There's a "<jsp:body" before the ETag
 
             // First, parse <jsp:attribute> elements:
             parseNamedAttributes(parent);
@@ -981,66 +986,10 @@ class Parser implements TagConstants {
         return result;
     }
 
-    /*
-     * <<<<<<< HEAD Params ::= `>' S? ( ( `<jsp:body>' ( ( S? Param+ S? `</jsp:body>' ) | <TRANSLATION_ERROR> ) ) |
-     * Param+ ) '</jsp:params>'
-     */
-    private void parseJspParams(Node parent) throws JasperException {
-        Node jspParamsNode = new Node.ParamsAction(start, parent);
-        parseOptionalBody(jspParamsNode, "jsp:params", JAVAX_BODY_CONTENT_PARAM);
-    }
-
-    /*
-     * Fallback ::= '/>' | ( `>' S? `<jsp:body>' ( ( S? ( Char* - ( Char* `</jsp:body>' ) ) `</jsp:body>' S? ) |
-     * <TRANSLATION_ERROR> ) `</jsp:fallback>' ) | ( '>' ( Char* - ( Char* '</jsp:fallback>' ) ) '</jsp:fallback>' )
-     */
-    private void parseFallBack(Node parent) throws JasperException {
-        Node fallBackNode = new Node.FallBackAction(start, parent);
-        parseOptionalBody(fallBackNode, "jsp:fallback", JAVAX_BODY_CONTENT_TEMPLATE_TEXT);
-    }
-
-    /*
-     * For Plugin: StdActionContent ::= Attributes PluginBody
-     *
-     * PluginBody ::= EmptyBody | ( '>' S? ( '<jsp:attribute' NamedAttributes )? '<jsp:body' ( JspBodyPluginTags |
-     * <TRANSLATION_ERROR> ) S? ETag ) | ( '>' S? PluginTags ETag )
-     *
-     * EmptyBody ::= '/>' | ( '>' ETag ) | ( '>' S? '<jsp:attribute' NamedAttributes ETag )
-     *
-     */
-    private void parsePlugin(Node parent) throws JasperException {
-        Attributes attrs = parseAttributes();
-        reader.skipSpaces();
-
-        Node pluginNode = new Node.PlugIn(attrs, start, parent);
-
-        parseOptionalBody(pluginNode, "jsp:plugin", JAVAX_BODY_CONTENT_PLUGIN);
-    }
-
-    /*
-     * PluginTags ::= ( '<jsp:params' Params S? )? ( '<jsp:fallback' Fallback? S? )?
-     */
-    private void parsePluginTags(Node parent) throws JasperException {
-        reader.skipSpaces();
-
-        if (reader.matches("<jsp:params")) {
-            parseJspParams(parent);
-            reader.skipSpaces();
-        }
-
-        if (reader.matches("<jsp:fallback")) {
-            parseFallBack(parent);
-            reader.skipSpaces();
-        }
-    }
-
     /*
      * StandardAction ::= 'include' StdActionContent | 'forward' StdActionContent | 'invoke' StdActionContent | 'doBody'
      * StdActionContent | 'getProperty' StdActionContent | 'setProperty' StdActionContent | 'useBean' StdActionContent |
-     * 'plugin' StdActionContent | 'element' StdActionContent ======= StandardAction ::= 'include' StdActionContent |
-     * 'forward' StdActionContent | 'invoke' StdActionContent | 'doBody' StdActionContent | 'getProperty'
-     * StdActionContent | 'setProperty' StdActionContent | 'useBean' StdActionContent | 'plugin' StdActionContent |
-     * 'element' StdActionContent >>>>>>> 76d2ad0a5d (Code clean-up. Formatting. No functional change.)
+     * 'plugin' StdActionContent | 'element' StdActionContent
      */
     private void parseStandardAction(Node parent) throws JasperException {
         Mark start = reader.mark();
@@ -1065,18 +1014,12 @@ class Parser implements TagConstants {
             parseSetProperty(parent);
         } else if (reader.matches(USE_BEAN_ACTION)) {
             parseUseBean(parent);
-        } else if (reader.matches(PLUGIN_ACTION)) {
-            parsePlugin(parent);
         } else if (reader.matches(ELEMENT_ACTION)) {
             parseElement(parent);
         } else if (reader.matches(ATTRIBUTE_ACTION)) {
             err.jspError(start, "jsp.error.namedAttribute.invalidUse");
         } else if (reader.matches(BODY_ACTION)) {
             err.jspError(start, "jsp.error.jspbody.invalidUse");
-        } else if (reader.matches(FALLBACK_ACTION)) {
-            err.jspError(start, "jsp.error.fallback.invalidUse");
-        } else if (reader.matches(PARAMS_ACTION)) {
-            err.jspError(start, "jsp.error.params.invalidUse");
         } else if (reader.matches(PARAM_ACTION)) {
             err.jspError(start, "jsp.error.param.invalidUse");
         } else if (reader.matches(OUTPUT_ACTION)) {
@@ -1182,7 +1125,7 @@ class Parser implements TagConstants {
         // 'CustomActionJSPContent', or 'CustomActionScriptlessContent'.
         // depending on body-content in TLD.
 
-        // Looking for a body, it still can be empty; but if there is
+        // Looking for a body, it still can be empty; but if there is a
         // a tag body, its syntax would be dependent on the type of
         // body content declared in the TLD.
         String bc;
@@ -1192,7 +1135,7 @@ class Parser implements TagConstants {
             bc = tagFileInfo.getTagInfo().getBodyContent();
         }
 
-        Node tagNode;
+        Node tagNode = null;
         if (tagInfo != null) {
             tagNode = new Node.CustomTag(tagName, prefix, shortTagName, uri, attrs, start, parent, tagInfo,
                     tagHandlerClass);
@@ -1532,12 +1475,6 @@ class Parser implements TagConstants {
             if (!reader.matchesETag(tag)) {
                 err.jspError(start, "jasper.error.emptybodycontent.nonempty", tag);
             }
-        } else if (bodyType == JAVAX_BODY_CONTENT_PLUGIN) {
-            // (note the == since we won't recognize JAVAX_* from outside this module).
-            parsePluginTags(parent);
-            if (!reader.matchesETag(tag)) {
-                err.jspError(reader.mark(), "jsp.error.unterminated", "&lt;" + tag);
-            }
         } else if (bodyType.equalsIgnoreCase(TagInfo.BODY_CONTENT_JSP) ||
                 bodyType.equalsIgnoreCase(TagInfo.BODY_CONTENT_SCRIPTLESS) || (bodyType == JAVAX_BODY_CONTENT_PARAM) ||
                 (bodyType == JAVAX_BODY_CONTENT_TEMPLATE_TEXT)) {
@@ -1643,10 +1580,6 @@ class Parser implements TagConstants {
             if ("beanName".equals(name)) {
                 return TagInfo.BODY_CONTENT_JSP;
             }
-        } else if (n instanceof Node.PlugIn) {
-            if ("width".equals(name) || "height".equals(name)) {
-                return TagInfo.BODY_CONTENT_JSP;
-            }
         } else if (n instanceof Node.ParamAction) {
             if ("value".equals(name)) {
                 return TagInfo.BODY_CONTENT_JSP;
